"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var app_background_1 = require("app/app.background");
var log_1 = require("app/log");
var request_set_1 = require("lib/request-set");
var listener_factories_1 = require("lib/utils/listener-factories");
var uriService = app_background_1.rp.services.uri;
var logGettingSavedRequests = log_1.log.extend({
    enabledCondition: { type: "C", C: "LOG_GETTING_SAVED_REQUESTS" },
    level: "all",
    name: "getting saved requests",
});
var requestSets = {
    allowedRequests: new request_set_1.RequestSet(),
    rejectedRequests: new request_set_1.RequestSet(),
};
var _a = listener_factories_1.createListenersMap([
    "onRequest",
]), eventTargets = _a.interfaces, eventListenersMap = _a.listenersMap;
function removeSavedRequestsByOriginURI(uri) {
    requestSets.allowedRequests.removeOriginUri(uri);
    requestSets.rejectedRequests.removeOriginUri(uri);
}
function notifyNewRequest(_a) {
    var originUri = _a.originUri, destUri = _a.destUri, isAllowed = _a.isAllowed, requestResult = _a.requestResult, unforbidable = _a.unforbidable, isInsert = _a.isInsert;
    if (isAllowed) {
        if (!unforbidable) {
            if (!isInsert) {
                removeSavedRequestsByOriginURI(destUri);
            }
            requestSets.rejectedRequests.removeRequest(originUri, destUri);
            requestSets.allowedRequests.addRequest(originUri, destUri, requestResult);
        }
    }
    else {
        requestSets.rejectedRequests.addRequest(originUri, destUri, requestResult);
        requestSets.allowedRequests.removeRequest(originUri, destUri);
    }
    eventListenersMap.onRequest.emit({
        destUri: destUri,
        isAllowed: isAllowed,
        originUri: originUri,
        requestResult: requestResult,
    });
}
function getRequestsHelper(currentlySelectedOrigin, allRequestsOnDocument, isAllowed) {
    var result = new request_set_1.RequestSet();
    var requests = allRequestsOnDocument.getAll();
    for (var originUri in requests) {
        if (uriService.getBaseDomain(originUri) !== currentlySelectedOrigin) {
            continue;
        }
        logGettingSavedRequests.log("test originUri: " + originUri);
        var originUriRequests = requests[originUri];
        for (var destBase in originUriRequests) {
            logGettingSavedRequests.log("test destBase: " + destBase);
            var destBaseRequests = originUriRequests[destBase];
            for (var destIdent in destBaseRequests) {
                logGettingSavedRequests.log("test destIdent: " + destIdent);
                var destIdentRequests = destBaseRequests[destIdent];
                for (var destUri in destIdentRequests) {
                    logGettingSavedRequests.log("test destUri: " + destUri);
                    var dest = destIdentRequests[destUri];
                    for (var i in dest) {
                        if (isAllowed === dest[i].isAllowed) {
                            result.addRequest(originUri, destUri, dest[i]);
                        }
                    }
                }
            }
        }
    }
    return result;
}
function _addRecursivelyAllRequestsFromURI(originURI, reqSet, checkedOrigins) {
    logGettingSavedRequests.log("Looking for other origins within allowed requests from " +
        originURI);
    if (!checkedOrigins[originURI]) {
        checkedOrigins[originURI] = true;
    }
    _addAllDeniedRequestsFromURI(originURI, reqSet);
    var allowedRequests = requestSets.allowedRequests.getOriginUri(originURI);
    if (allowedRequests) {
        for (var destBase in allowedRequests) {
            for (var destIdent in allowedRequests[destBase]) {
                for (var destURI in allowedRequests[destBase][destIdent]) {
                    logGettingSavedRequests.log("Found allowed request to <" + destURI + "> " +
                        "from <" + originURI + ">");
                    reqSet.addRequest(originURI, destURI, allowedRequests[destBase][destIdent][destURI]);
                    if (!checkedOrigins[destURI]) {
                        checkedOrigins[destURI] = true;
                        _addRecursivelyAllRequestsFromURI(destURI, reqSet, checkedOrigins);
                    }
                }
            }
        }
    }
}
function _addAllDeniedRequestsFromURI(originUri, reqSet) {
    logGettingSavedRequests.log("Looking for other origins within denied requests from " + originUri);
    var requests = requestSets.rejectedRequests.getOriginUri(originUri);
    if (requests) {
        for (var destBase in requests) {
            for (var destIdent in requests[destBase]) {
                for (var destUri in requests[destBase][destIdent]) {
                    logGettingSavedRequests.log("Found denied request to <" + destUri + "> " +
                        ("from <" + originUri + ">"));
                    reqSet.addRequest(originUri, destUri, requests[destBase][destIdent][destUri]);
                }
            }
        }
    }
}
function getDeniedRequests(currentlySelectedOrigin, allRequestsOnDocument) {
    logGettingSavedRequests.log("## getDeniedRequests");
    return getRequestsHelper(currentlySelectedOrigin, allRequestsOnDocument, false);
}
function getAllowedRequests(currentlySelectedOrigin, allRequestsOnDocument) {
    logGettingSavedRequests.log("## getAllowedRequests");
    return getRequestsHelper(currentlySelectedOrigin, allRequestsOnDocument, true);
}
function getAllRequestsInBrowser(browser) {
    var reqSet = new request_set_1.RequestSet();
    var uri = uriService.stripFragment(browser.currentURI.spec);
    _addRecursivelyAllRequestsFromURI(uri, reqSet, {});
    return reqSet;
}
exports.Requests = {
    _removeSavedRequestsByOriginURI: removeSavedRequestsByOriginURI,
    _requestSets: requestSets,
    getAllRequestsInBrowser: getAllRequestsInBrowser,
    getAllowedRequests: getAllowedRequests,
    getDeniedRequests: getDeniedRequests,
    notifyNewRequest: notifyNewRequest,
    onRequest: eventTargets.onRequest,
};